/*
     This file is part of GNUnet

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/
/**
 * @file src/applications/afs/gtkui/saveas.c
 * @author Christian Grothoff
 * @author Igor Wronsky
 *
 * This code opens a save-as window.
 **/

#include "config.h"
#include <gtk/gtk.h>

#include "util/logging.h"
#include "util/xmalloc.h"
#include "applications/afs/gtkui/download.h"
#include "applications/afs/gtkui/helper.h"

/**
 * Destroy the download model 
 *
 * Notes: everything allocated to *dlm in download.c
 * should be freed there, not here (problems otherwise). :( 
 * This routine will be called by destructive cascade at last
 * when its sure dlm is no longer needed.
 *
 * @param dlm the DownloadModel to be destroyed
 **/
static void destroyDLM(GtkWidget * widget,
                       DownloadModel * dlm) {
  LOG(LOG_DEBUG, "DEBUG: Destroying dlm (%d)\n", dlm);
  FREENONNULL(dlm->fileName);
  FREE(dlm);
}

void thawList(GtkWidget * dummy, 
	      DownloadModel * dlm) {
  gtk_clist_thaw(GTK_CLIST(dlm->listModel->search_result_list));
  destroyWidget(NULL, dlm->saveAsWindow);
}
 
/**
 * Get the selected filename and start downloading 
 *
 * @param dlm Data required for the download
 **/
void file_ok_sel(GtkWidget     * w,
		 DownloadModel * dlm) {
  gchar * filename;

  gtk_clist_remove(GTK_CLIST(dlm->listModel->search_result_list),
  		   dlm->listModel->selectedRow);
  dlm->listModel->selectedRow = -1;
  gtk_clist_thaw(GTK_CLIST(dlm->listModel->search_result_list));

  filename
    = gtk_file_selection_get_filename(GTK_FILE_SELECTION(dlm->saveAsWindow));
  dlm->fileName = STRDUP(filename);
  startDownload(dlm);
  gtk_widget_hide(dlm->saveAsWindow);
}

/**
 * Open the window that prompts the user for the 
 * filename.
 * This method must open the window,
 * copy the arguments and return. After the method
 * returns, the arguments passed to it will be
 * freed, so pointer should not be retained.
 * The method executes during a signal handler,
 * so a GTK lock is not required to to GUI 
 * operations.
 * 
 * @param dlm Data required for the download
 **/
void openSaveAs(DownloadModel * dlm) {
  GtkWidget * window;
 
  window = gtk_file_selection_new("save as");
  dlm->saveAsWindow = window;
  if (dlm->root.header.filename != NULL)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(window),
				    dlm->root.header.filename);
  gtk_signal_connect(GTK_OBJECT(window), "destroy",
		     (GtkSignalFunc) destroyDLM,
		     dlm);
  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(window)->ok_button),
		     "clicked", (GtkSignalFunc) file_ok_sel, 
		     dlm);
  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(window)->cancel_button),
		     "clicked", 
		     (GtkSignalFunc) thawList, 
		     dlm);
  gtk_widget_show(window);
}

/* end of saveas.c */
