/*
     This file is part of GNUnet

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * The most message types for the GNUnet AFS.
 * @file include/applications/afs/protocol/messages.h
 * @author Christian Grothoff
 **/

#ifndef MESSAGES_H
#define MESSAGES_H

#include "config.h"

#include "server/coreapi.h"
#include "applications/afs/encoding/block.h"
#include "applications/afs/database/lookup.h"


/**
 * by which amount do we decrement the TTL for simple forwarding /
 * indirection of the query; in milli-seconds.  Set somewhat in
 * accordance to your network latency (above the time it'll take you
 * to send a packet and get a reply).
 **/
#define TTL_DECREMENT 5 * cronSECONDS

/* *********** STRUCTS for the p2p protocol *********** */

/**
 * Request for content. The number of queries can
 * be determined from the header size.
 **/
typedef struct {
  p2p_HEADER header; 

  /**
   * How important is this request (network byte order) 
   **/
  int priority;         

  /**
   * Time to live in cronMILLIS (network byte order)  
   **/
  int ttl;              

  /**
   * To whom to return results? 
   **/
  HostIdentity returnTo;

  /**
   * Hashcodes of the file(s) we're looking for. If multiple queries
   * are given, the first query is the super-query for the bloom
   * filter. If only one query is given, the bloom filter should NOT
   * be used since it does not contain summaries for simple 1k
   * blocks. It is not possible to group multiple queries with this
   * message type if they are not dominated by the same super-query.
   **/
  HashCode160 queries[0]; 

} AFS_p2p_QUERY;

/**
 * Return message for search result (with double-hash proof).
 **/
typedef struct {
  p2p_HEADER header;   

  /**
   * The double-hash
   **/
  HashCode160 hash;

  /**
   * The search result.
   **/
  RootNode result;

} AFS_p2p_3HASH_RESULT;

/**
 * Return message for content download (CHK style)
 **/
typedef struct {
  p2p_HEADER header;   

  /**
   * The search result.
   **/
  CONTENT_Block result;

} AFS_p2p_CHK_RESULT;

/* ************************* CS messages ********************** */
/* these messages are exchanged between gnunetd and the clients */

/**
 * TCP communication: search result content send back by gnunetd
 **/
typedef struct {

  /**
   * The TCP header (values: sizeof(AFS_CS_RESULT_3HASH), AFS_CS_PROTO_RESULT_3HASH) 
   **/ 
  CS_HEADER header;

  /**
   * The double-hash 
   **/
  HashCode160 hash;

  /**
   * The search result.
   **/
  RootNode result;

} AFS_CS_RESULT_3HASH;

/* NOTE: AFS_CS_QUERY and AFS_CS_RESULT_CHK
   had to be defined in encoding/block.h */

/**
 * Structure for an incoming request messages from the local TCP link
 * to add content to the node.
 **/
typedef struct {

  /**
   * The TCP header (values: sizeof(AFS_CS_INSERT_CHK), AFS_CS_PROTO_INSERT_CHK) 
   **/ 
  CS_HEADER header; 

  /**
   * The (initial) priority of the data  (network byte order)
   **/
  unsigned int importance;

  /**
   * The data to insert 
   **/
  CONTENT_Block content;  

} AFS_CS_INSERT_CHK;

/**
 * Structure for an incoming request messages from the local TCP link
 * to add content to the node.
 **/
typedef struct {

  /**
   * The TCP header (values: sizeof(AFS_CS_INSERT_3HASH), AFS_CS_PROTO_INSERT_3HASH) 
   **/ 
  CS_HEADER header; 

  /**
   * The (initial) priority of the data  (network byte order)
   **/
  unsigned int importance;

  /**
   * The doubleHash of the plaintext.
   **/
  HashCode160 doubleHash;

  /**
   * The data to insert 
   **/
  CONTENT_Block content;  

} AFS_CS_INSERT_3HASH;

/**
 * Structure for an incoming request messages from the local TCP link
 * to add content to the INDEX of the node.
 **/
typedef struct {

  /**
   * The TCP header (values: sizeof(AFS_CS_INDEX_BLOCK), AFS_CS_PROTO_INDEX_BLOCK) 
   **/ 
  CS_HEADER header; 

  /**
   * indexing information 
   **/
  ContentIndex contentIndex;

} AFS_CS_INDEX_BLOCK;

#define CS_FILE_LIST_FILENAME 1024

/**
 * Structure for an incoming request messages from the local TCP link
 * to add a filename to the list of directly shared files
 **/
typedef struct {

  /**
   * The TCP header (values: sizeof(AFS_CS_INDEX_FILE), AFS_CS_PROTO_INDEX_FILE) 
   **/ 
  CS_HEADER header; 

  /**
   * the filename (max. 1023 characters), padded with zeros 
   **/
  char filename[CS_FILE_LIST_FILENAME];

} AFS_CS_INDEX_FILE;

/**
 * Structure for an incoming request messages from the local TCP link
 * to add a super-query to the bloom filter.
 **/
typedef struct {

  /**
   * The TCP header (values: sizeof(AFS_CS_INDEX_SUPER), AFS_CS_PROTO_INDEX_SUPER) 
   **/ 
  CS_HEADER header; 

  /**
   * Is this really a super-hash or is it the top-CHK that
   * should be "extraordinarily" inserted into the search-filter?
   **/
  int isSuper;

  /**
   * The super-hash for the bloom-filter.
   **/
  HashCode160 superHash;

} AFS_CS_INDEX_SUPER;

#endif
